#!/usr/bin/env python
# coding: utf-8

#%%
import numpy as np
import rasterio
import stac
from matplotlib import pyplot

#%%
s = stac.STAC('http://brazildatacube.dpi.inpe.br/stac/', False)

#%%
s.catalog

#%%
collection = s.collection('CB4_64_16D_STK-1')
collection

#%%
items = collection.get_items()
items

#%%
items.features[0].assets

#%%
red = items.features[0].assets['BAND15'].download()
green = items.features[0].assets['BAND14'].download()
blue = items.features[0].assets['BAND13'].download()

red

#%%
r = rasterio.open(red).read(1)
g = rasterio.open(green).read(1)
b = rasterio.open(blue).read(1)


#%%
r.max()

#%%
def normalize(array):
    """Normalizes numpy arrays into scale 0.0 - 1.0"""
    array_min, array_max = array.min(), array.max()
    return ((array - array_min)/(array_max - array_min))

#%%
rgb = np.dstack((normalize(r), normalize(g), normalize(b)))
pyplot.imshow(rgb)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""STAC Item module."""

import json
import os
import shutil
from collections.abc import Iterable
from urllib.parse import urlparse

import requests
from pkg_resources import resource_string

from .common import Link, Provider
from .utils import Utils


class Asset(dict):
    """Asset object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Asset metadata.
        """
        super(Asset, self).__init__(data or {})

    @property
    def href(self):
        """:return: the Asset href."""
        return self['href']

    @property
    def title(self):
        """:return: the Asset title."""
        return self['title']

    @property
    def type(self):
        """:return: the Asset type."""
        return self['type']

    def download(self, dir=None): # pragma: no cover
        """Download the asset to an indicated folder.

        If tqdm is installed a progressbar will be shown.

        :param dir: Directory path to download the asset, if left None,
                    the asset will be downloaded to the current
                    working directory.
        :return: path to downloaded file.
        """
        filename = urlparse(self['href'])[2].split('/')[-1]

        if dir:
            filename = os.path.join(dir, filename)
            os.makedirs(os.path.dirname(filename), exist_ok=True)

        response = requests.get(self['href'], stream=True)

        response.raise_for_status()

        try:
            from tqdm import tqdm

            with tqdm.wrapattr(open(filename, 'wb'), 'write', miniters=1,
                        total=int(response.headers.get('content-length', 0)),
                        desc=filename) as fout:
                for chunk in response.iter_content(chunk_size=4096):
                    fout.write(chunk)

        except ImportError:
            with open(filename, 'wb') as f:
                shutil.copyfileobj(response.raw, f)

        return filename


class Geometry(dict):
    """The Geometry Object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Geometry metadata.
        """
        super(Geometry, self).__init__(data or {})

    @property
    def type(self):
        """:return: the Geometry type."""
        return self['type']

    @property
    def coordinates(self):
        """:return: the Geometry coordinates."""
        return self['coordinates']

class Properties(dict):
    """The Properties Object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Properties metadata.
        """
        super(Properties, self).__init__(data or {})
        self._providers = [Provider(p) for p in self['providers']] if 'providers' in self else []

    @property
    def datetime(self):
        """:return: the datetime property."""
        return self['datetime']

    @property
    def license(self):
        """:return: the license property."""
        return self['license']

    @property
    def providers(self):
        """:return: the providers property."""
        return self._providers

    @property
    def title(self):
        """:return: the title property."""
        return self['title']

    @property
    def created(self):
        """:return: the created property."""
        return self['created']

    @property
    def updated(self):
        """:return: the updated property."""
        return self['updated']

class Item(dict):
    """The GeoJSON Feature of a STAC Item."""

    def __init__(self, data, validate=False):
        """Initialize instance with dictionary data.

        :param data: Dict with Item metadata.
        :param validate: true if the Item should be validate using its jsonschema. Default is False.
        """
        self._validate = validate
        super(Item, self).__init__(data or {})

        self._schema = json.loads(resource_string(__name__, f'jsonschemas/{self.stac_version}/item.json'))

        if self._validate:
            Utils.validate(self)

        self._assets = {key: Asset(value) for key,value in self['assets'].items()} if 'assets' in self else {}
        self._links = [Link(link) for link in self['links']] if 'links' in self else []

    @property
    def stac_version(self):
        """:return: the STAC version."""
        return self['stac_version'] if 'stac_version' in self else '0.7.0'

    @property
    def id(self):
        """:return: the Item identifier."""
        return self['id']

    @property
    def type(self):
        """:return: the Item type."""
        return self['type']

    @property
    def bbox(self):
        """:return: the Item Bounding Box."""
        return self['bbox']

    @property
    def collection(self):
        """:return: the Item Collection."""
        return self['collection']

    @property
    def geometry(self):
        """:return: the Item Geometry."""
        return Geometry(self['geometry'])

    @property
    def properties(self):
        """:return: the Item properties."""
        return Properties(self['properties'])

    @property
    def links(self):
        """:return: the Item related links."""
        return self._links

    @property
    def assets(self):
        """:return: the Item related assets."""
        return self._assets

    @property
    def schema(self):
        """:return: the Collection jsonschema."""
        return self._schema

    def _repr_html_(self): # pragma: no cover
        """HTML repr."""
        return Utils.render_html('item.html', item=self)

    def download(self, dir=None): # pragma: no cover
        """Download an asset given a band name.

        :param dir: Directory path to download the asset, if left None,
                    the asset will be downloaded to the current
                    working directory.
        :return: path to downloaded file.
        """
        output = dict()
        for asset_name, asset in self.assets.items():
            output[asset_name] = asset.download(dir=dir)

        return output

    def read(self, band_name, window=None, bbox=None, crs=None): # pragma: no cover
        """Read an asset given a band name.

        Notes:
            You must install the extra `geo` containing the `rasterio` and `Shapely` library
            in order to use this method:

                pip install stac.py[geo]

        :param band_name: Band name used in the asset
        :type band_name: str
        :param window: window crop
        :type window: raster.windows.Window
        :param bbox: The bounding box
        :type bbox: Union[str,Tuple[float],List[float],BaseGeometry]
        :param crs: The Coordinate Reference System
        :return: the asset as a numpy array
        :rtype: numpy.ndarray
        """
        import rasterio
        from rasterio.crs import CRS
        from rasterio.warp import transform
        from rasterio.windows import from_bounds

        with rasterio.open(self.assets[band_name]['href']) as dataset:
            if bbox:
                bbox = Utils.build_bbox(bbox)

                w, s, e, n = bbox.bounds

                source_crs = CRS.from_string('EPSG:4326')

                if crs:
                    source_crs = CRS.from_string(crs)

                t = transform(source_crs, dataset.crs, [w, e], [s, n])
                window = from_bounds(t[0][0], t[1][0], t[0][1], t[1][1], dataset.transform)

            asset = dataset.read(1, window=window)

        return asset

class ItemCollection(dict):
    """The GeoJSON Feature Collection of STAC Items."""

    def __init__(self, data, validate=False):
        """Initialize instance with dictionary data.

        :param data: Dict with Item Collection metadata.
        :param validate: true if the Item Collection should be validate using its jsonschema. Default is False.
        """
        self._validate = validate
        super(ItemCollection, self).__init__(data or {})

        self._features = [Item(i, self._validate) for i in self['features']] if 'features' in self else []
        self._links = [Link(i) for i in self['links']] if 'links' in self else []

    @property
    def type(self):
        """:return: the Item Collection type."""
        return self['type']

    @property
    def features(self):
        """:return: the Item Collection list of GeoJSON Features."""
        return self._features

    @property
    def links(self):
        """:return: the Item Collection list of GeoJSON Features."""
        return self._links

    def _repr_html_(self): # pragma: no cover
        """HTML repr."""
        return Utils.render_html('itemcollection.html', itemcollection=self)

    def __iter__(self): # pragma: no cover
        """Feature iterator."""
        return self.features.__iter__()

    def __next__(self): # pragma: no cover
        """Next Feature iterator."""
        return next(self.features)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Unit-test for stac.py."""
from .test_stac import TestCli, TestStac, TestUtils



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""STAC Collection module."""
import json

from pkg_resources import resource_string

from .catalog import Catalog
from .common import Provider
from .item import Item, ItemCollection
from .utils import Utils


class Stats(dict):
    """The Stats object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Stats metadata.
        """
        super(Stats, self).__init__(data or {})

    @property
    def min(self):
        """:return: the min of Stats for a collection."""
        return self['min']

    @property
    def max(self):
        """:return: the max of Stats for a collection."""
        return self['max']

class SpatialExtent(dict):
    """The Spatial Extent object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Spatial Extent metadata.
        """
        super(SpatialExtent, self).__init__(data or {})

    @property
    def bbox(self):
        """:return: the bbox of the Spatial Extent."""
        return self['bbox']

class TemporalExtent(dict):
    """The Temporal Extent object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Temporal Extent metadata.
        """
        super(TemporalExtent, self).__init__(data or {})

    @property
    def interval(self):
        """:return: the interval of the Temporal Extent."""
        return self['interval']

class Extent(dict):
    """The Extent object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Extent metadata.
        """
        super(Extent, self).__init__(data or {})

    @property
    def spatial(self):
        """:return: the spatial extent."""
        if 'bbox' in self['spatial']:
            return SpatialExtent(self['spatial'])
        return self['spatial']

    @property
    def temporal(self):
        """:return: the temporal extent."""
        if 'interval' in self['temporal']:
            return TemporalExtent(self['temporal'])
        return self['temporal']


class Collection(Catalog):
    """The STAC Collection."""

    def __init__(self, data, validate=False):
        """Initialize instance with dictionary data.

        :param data: Dict with collection metadata.
        :param validate: true if the Collection should be validate using its jsonschema. Default is False.
        """
        self._validate = validate
        super(Collection, self).__init__(data or {}, validate)

        self._schema = json.loads(resource_string(__name__, f'jsonschemas/{self.stac_version}/collection.json'))

        if self._validate:
            Utils.validate(self)

        self._summaries = {k: Stats(v) for k, v in self['summaries'].items()} if 'summaries' in self else {}
        self._providers = [Provider(provider) for provider in self['providers']] if 'providers' in self else []


    @property
    def keywords(self):
        """:return: the Collection list of keywords."""
        return self['keywords']

    @property
    def version(self):
        """:return: the Collection version."""
        return self['version']

    @property
    def license(self):
        """:return: the Collection license."""
        return self['license']

    @property
    def providers(self):
        """:return: the Collection list of providers."""
        return self._providers
    @property
    def extent(self):
        """:return: the Collection extent."""
        return Extent(self['extent'])

    @property
    def properties(self):
        """:return: the Collection properties."""
        return self['properties']

    @property
    def summaries(self):
        """:return: the Collection summaries."""
        return self._summaries

    @property
    def schema(self):
        """:return: the Collection jsonschema."""
        return self._schema

    def get_items(self, item_id=None, filter=None):
        """Retrieve items of the collection.

        :param item_id: (optional) a str with a STAC Item id.
        :type item_id: str

        :param filter: (optional) A dictionary with valid STAC query parameters.
        :type filter: dict

        :return: A GeoJSON FeatureCollection of STAC Items from the collection.
        """
        if filter is not None and 'bbox' in filter:
            filter['bbox'] = Utils.build_bbox_as_str(filter['bbox'])

        for link in self['links']:
            if link['rel'] == 'items':
                if item_id is not None:
                    data = Utils._get(f'{link["href"]}/{item_id}')
                    return Item(data, self._validate)
                data = Utils._get(f'{link["href"]}', params=filter)
                return ItemCollection(data)
        return ItemCollection({})

    def _repr_html_(self): # pragma: no cover
        """HTML repr."""
        return Utils.render_html('collection.html', collection=self)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Version information for Python Client Library for STAC.

This file is imported by ``stac.__init__``,
and parsed by ``setup.py``.
"""


__version__ = '0.9.0-10'



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Common objects for STAC."""

class Link(dict):
    """Link object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Link metadata.
        """
        super(Link, self).__init__(data or {})

    @property
    def rel(self):
        """:return: the Link relation."""
        return self['rel']

    @property
    def href(self):
        """:return: the Link url."""
        return self['href']

    @property
    def type(self):
        """:return: the type of the Link object."""
        return self['type']

    @property
    def title(self):
        """:return: the title of the Link object."""
        return self['title']

class Provider(dict):
    """The Provider Object."""

    def __init__(self, data):
        """Initialize instance with dictionary data.

        :param data: Dict with Provider metadata.
        """
        super(Provider, self).__init__(data or {})

    @property
    def name(self):
        """:return: the Provider name."""
        return self['name']

    @property
    def description(self):
        """:return: the Provider description."""
        return self['description']

    @property
    def roles(self):
        """:return: the Provider roles."""
        return self['roles']

    @property
    def url(self):
        """:return: the Provider url."""
        return self['url']



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Python API client wrapper for STAC."""
import warnings

from requests import HTTPError

from .catalog import Catalog
from .collection import Collection
from .item import ItemCollection
from .utils import Utils


class STAC:
    """This class implements a Python API client wrapper for STAC.

    See https://github.com/radiantearth/stac-spec for more information on STAC.

    :param url: URL for the Root STAC Catalog.
    :type url: str
    """

    def __init__(self, url, validate=False, access_token=None):
        """Create a STAC client attached to the given host address (an URL).

        :param url: URL for the Root STAC Catalog.
        :type url: str
        :param validate: True if responses should ve validated
        :type validate: bool
        :param access_token: Authentication for the STAC API
        :type access_token: str
        """
        self._url = url
        self._collections = dict()
        self._catalog = dict()
        self._validate = validate
        self._access_token = f'?access_token={access_token}' if access_token else ''

    @property
    def conformance(self): # pragma: no cover
        """Return the list of conformance classes that the server conforms to."""
        return Utils._get('{}/conformance'.format(self._url))

    @property
    def catalog(self):
        """
        Retrieve the available collections in the STAC Catalog.

        :return list of available collections.
        """
        if len(self._collections) > 0:
            return list(self._collections.keys())

        url = f'{self._url}{self._access_token}'
        response = Utils._get(url)

        self._catalog = Catalog(response, self._validate)

        for i in self._catalog.links:
            if i.rel == 'child':
                if '?' in i.href:  # pragma: no cover
                    collection_name = i.href.split('/')[-1]
                    self._collections[collection_name[:collection_name.index('?')]] = None
                else:
                    self._collections[i.href.split('/')[-1]] = None
        return list(self._collections.keys())


    @property
    def collections(self):
        """Return all available collections.

        :returns: A dict containing all collections.
        :rype: dict
        """
        url = '/'.join(self._url.split('/')[:-1]) if self._url.endswith('/stac') else self._url
        data = Utils._get(f'{url}/collections{self._access_token}')
        self._collections = {collection['id']: Collection(collection, self._validate) for collection in data['collections']}

        return self._collections


    def collection(self, collection_id):
        """Return the given collection.

        :param collection_id: A str for a given collection_id.
        :type collection_id: str

        :returns: A STAC Collection.
        :rtype: dict
        """
        if collection_id in self._collections.keys() and \
            self._collections[collection_id] is not None:
            return self._collections[collection_id]
        try:
            url = '/'.join(self._url.split('/')[:-1]) if self._url.endswith('/stac') else self._url
            data = Utils._get(f'{url}/collections/{collection_id}{self._access_token}')
            self._collections[collection_id] = Collection(data, self._validate)
        except HTTPError as e:
            raise KeyError(f'Could not retrieve information for collection: {collection_id}')
        return self._collections[collection_id]


    def search(self, filter=None):
        """Retrieve Items matching a filter.

        :param filter: (optional) A dictionary with valid STAC query parameters.
        :type filter: dict

        :returns: A GeoJSON FeatureCollection.
        :rtype: dict
        """
        if not self._catalog:  # pragma: no cover
            self.catalog

        url = f'{self._url}/search{self._access_token}'

        if filter is not None and 'bbox' in filter:
            filter['bbox'] = Utils.build_bbox_as_str(filter['bbox'])

        data = Utils._get(url, params=filter)
        return ItemCollection(data, self._validate)

    @property
    def url(self):
        """Return the STAC server instance URL."""
        return self._url

    def __repr__(self):
        """Return the string representation of a STAC object."""
        text = 'stac("{}")'.format(self.url)
        return text

    def _repr_html_(self): # pragma: no cover
        """HTML repr."""
        collections = str()
        for collection in self.catalog:
            collections += f"<li>{collection}</li>"
        return f"""<p>STAC</p>
                    <ul>
                     <li><b>URL:</b> {self._url}</li>
                     <li><b>Collections:</b></li>
                     <ul>
                     {collections}
                     </ul>
                   </ul>
               """

    def __str__(self):
        """Return the string representation of a STAC object."""
        return '<STAC [{}]>'.format(self.url)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Command line interface for the STAC client."""
import json

import click

from .stac import STAC


@click.group()
def cli():
    """STAC on command line."""
    pass #pragma: no cover

@click.command()
@click.option('--url', default=None, help='The STAC server address (an URL).')
@click.option('--access-token', default=None, help='Personal Access Token of the BDC Auth')
def catalog(url, access_token=None):
    """Return the list of available collections in the catalog."""
    service = STAC(url, access_token=access_token)

    retval = service.catalog
    for c in retval:
        print(c)


@click.command()
@click.option('--url', default=None, help='The STAC server address (an URL).')
@click.option('--collection-id', default=None, help='The Collection id.')
@click.option('--access-token', default=None, help='Personal Access Token of the BDC Auth')
def collection(url, collection_id, access_token=None):
    """Return the metadata for a given collection."""
    service = STAC(url, access_token=access_token)

    retval = service.collection(collection_id)

    print(json.dumps(retval, indent=2))

@click.command()
@click.option('--url', default=None, help='The STAC server address (an URL).')
@click.option('--collection-id', default=None, help='The collection ID.')
@click.option('--limit', default=10, help='The maximum number of results to return (page size). Defaults to 10')
@click.option('--page', default=1, help='The page number of results. Defaults to 1. (STAC 0.7.0 only)')
@click.option('--datetime', help='Single date, date+time, or a range (\'/\' seperator), formatted to RFC 3339, section 5.6')
@click.option('--bbox', default=None, help='Requested bounding box west, south, east, north')
@click.option('--access-token', default=None, help='Personal Access Token of the BDC Auth')
def items(url, collection_id, limit, page, datetime, bbox, access_token=None):
    """Return items from a given collection ID and filters."""
    service = STAC(url, access_token=access_token)
    service.catalog
    filter = {
        'limit': limit,
        'page': page
    }

    if bbox is not None:
        filter['bbox'] = bbox
    if datetime is not None:
        if service._catalog.stac_version in ['0.8.0', '0.8.1']:
            filter['datetime'] = datetime
        elif service._catalog.stac_version == '0.7.0':
            filter['time'] = datetime

    retval = service.collection(collection_id).get_items(filter=filter)

    print(json.dumps(retval, indent=2))

@click.command()
@click.option('--url', default=None, help='The STAC server address (an URL).')
@click.option('--collections', default=None, help='Array of Collection IDs to include in the search for items.'\
              'Only Items in one of the provided Collections will be searched')
@click.option('--ids', default=None, help='Array of Item ids to return. All other filter parameters '\
                                          'that further restrict the number of search results '\
                                          '(except next and limit) are ignored')
@click.option('--intersects', default=None, help='Searches items by performing intersection between their '\
                                                 'geometry and provided GeoJSON Feature')
@click.option('--limit', default=10, help='The maximum number of results to return (page size). Defaults to 10')
@click.option('--next', default=None, help='The token to retrieve the next set of results, e.g., '\
                                        'offset, page, continuation token. (STAC 0.8.x only)')
@click.option('--page', default=1, help='The page number of results. Defaults to 1. (STAC 0.7.0 only)')
@click.option('--datetime', help='Single date, date+time, or a range (\'/\' seperator), formatted to RFC 3339, section 5.6')
@click.option('--bbox', default=None, help='Requested bounding box west, south, east, north')
@click.option('--access-token', default=None, help='Personal Access Token of the BDC Auth')
def search(url, collections, ids, intersects, limit, next, page, datetime, bbox, access_token=None):
    """Search through a STAC catalog."""
    service = STAC(url, access_token=access_token)
    service.catalog

    filter = {
        'limit': limit,
    }

    if service._catalog.stac_version == '0.8.0':
        if next is not None:
            filter['next'] = next
        if datetime is not None:
            filter['datetime'] = datetime
    elif service._catalog.stac_version == '0.7.0':
        if datetime is not None:
            filter['time'] = datetime


    if bbox is not None:
        filter['bbox'] = bbox
    if collections is not None:
        filter['collections'] = collections
    if intersects is not None:
        filter['intersects'] = intersects
    if ids is not None:
        filter['ids'] = ids

    retval = service.search(filter=filter)

    print(json.dumps(retval, indent=2))

cli.add_command(catalog)
cli.add_command(collection)
cli.add_command(items)
cli.add_command(search)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Utility data structures and algorithms."""

from collections.abc import Iterable

import jinja2
import requests
from jsonschema import RefResolver, validate
from pkg_resources import resource_filename

base_schemas_path = resource_filename(__name__, 'jsonschemas/')
templateLoader = jinja2.FileSystemLoader( searchpath=resource_filename(__name__, 'templates/'))
templateEnv = jinja2.Environment( loader=templateLoader )

class Utils:
    """Utils STAC object."""

    @staticmethod
    def _get(url, params=None):
        """Query the STAC service using HTTP GET verb and return the result as a JSON document.

        :param url: The URL to query must be a valid STAC endpoint.
        :type url: str

        :param params: (optional) Dictionary, list of tuples or bytes to send
        in the query string for the underlying `Requests`.
        :type params: dict

        :rtype: dict

        :raises ValueError: If the response body does not contain a valid json.
        """
        response = None

        if params is not None:
            if 'intersects' in params or 'query' in params:
                if 'collections' in params and isinstance(params['collections'], str):
                    params['collections'] = params['collections'].split(',')
                if 'ids' in params and isinstance(params['ids'], str):
                    params['ids'] = params['ids'].split(',')
                if 'bbox' in params and isinstance(params['bbox'], str):
                    params['bbox'] = [float(coord) for coord in params['bbox'].split(',')]

                response = requests.post(url, json=params)
            else:
                if 'collections' in params and isinstance(params['collections'], Iterable):
                    params['collections'] = ','.join(params['collections'])
                response = requests.get(url, params=params)
        else:
            response = requests.get(url)

        response.raise_for_status()

        content_type = response.headers.get('content-type')

        if content_type not in ('application/json', 'application/geo+json'):
            raise ValueError('HTTP response is not JSON: Content-Type: {}'.format(content_type))

        return response.json()


    @staticmethod
    def validate(stac_object):
        """Validate a STAC Object using its jsonschema.

        :raise ValidationError: raise a ValidationError if the STAC Object couldn't be validated.
        """
        resolver = RefResolver(f'file://{base_schemas_path}{stac_object.stac_version}/', None)

        validate(stac_object, stac_object.schema, resolver=resolver)

    @staticmethod
    def render_html(template_name, **kwargs): # pragma: no cover
        """Render Jinja2 HTML template."""
        template = templateEnv.get_template( template_name )
        return template.render(**kwargs)

    @staticmethod
    def build_bbox(bbox):
        """Define a common way to create the minimum bounding region.

        :param bbox: The bounding box
        :type bbox: Union[str,List[float],Tuple[float]]

        :returns shapely.geometry.base.BaseGeometry
        """
        from shapely.geometry import box
        from shapely.geometry.base import BaseGeometry

        if isinstance(bbox, str):
            try:
                bbox = [float(elm) for elm in bbox.split(',')]
            except ValueError:
                raise TypeError(f'Invalid bbox {bbox}')

        if isinstance(bbox, Iterable):
            bbox = box(*bbox)

        if not isinstance(bbox, BaseGeometry) or bbox.is_empty:
            raise TypeError(f'Invalid bbox {bbox}')

        return bbox

    @staticmethod
    def build_bbox_as_str(bbox) -> str:
        """Retrieve the string representation of a minimum bounding region.

        :param bbox: The bounding box

        :returns str
        """
        bounds = Utils.build_bbox(bbox).bounds

        return f'{bounds[0]},{bounds[1]},{bounds[2]},{bounds[3]}'



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Python Client Library for STAC."""

from . import cli
from .catalog import Catalog
from .collection import Collection, Extent, Provider
from .common import Link
from .item import Geometry, Item, ItemCollection
from .stac import STAC
from .utils import Utils
from .version import __version__

__all__ = ('__version__',
           'stac', )



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""Unit-test for STAC operations."""

import json
import os
import re
from pathlib import Path

import pytest
import requests
from click.testing import CliRunner
from pkg_resources import resource_filename, resource_string

import stac

url = os.environ.get('STAC_SERVER_URL', 'http://localhost')
match_url = re.compile(url)



@pytest.fixture
def requests_mock(requests_mock):
    requests_mock.get(re.compile('https://geojson.org/'), real_http=True)
    yield requests_mock

@pytest.fixture(scope='session')
def stac_objects():
    directory = resource_filename(__name__, 'jsons/')
    files = dict()
    for path in Path(directory).rglob('*.json'):
        path = str(path)
        s = path.split('/')

        file_path = '/'.join(s[-3:])
        file = json.loads(resource_string(__name__, file_path))
        if s[-2] in files:
            files[s[-2]][s[-1]] = file
        else:
            files[s[-2]] = {s[-1]: file}

    return files

@pytest.fixture(scope='module')
def runner():
    return CliRunner()

class TestUtils:
    def test_ok(self, requests_mock):
        requests_mock.get(match_url, json={"key":"value"}, status_code=200, headers={'content-type':'application/json'})
        stac.Utils._get(url)

    def test_error(self, requests_mock):
        requests_mock.get(match_url, json={"key":"value"}, status_code=200, headers={'content-type':'text/plain'})
        with pytest.raises(ValueError):
            stac.Utils._get(url)



class TestStac:
    def test_stac(self):
        s = stac.STAC(url, True)
        assert s.url == url
        assert repr(s) == f'stac("{url}")'
        assert str(s) == f'<STAC [{url}]>'

    def test_catalog(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)

            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            response = s.catalog
            assert s._catalog.stac_version
            assert s._catalog.id
            assert s._catalog.description
            assert s._catalog.title
            assert s._catalog.links[0].type
            assert s._catalog.links[0].title
            assert s._catalog.links[0].href
            assert s._catalog.links[0].rel
            assert response == ['my_collection1']

    def test_collections(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url,
                              json=dict(collections=[stac_objects[k]['collection.json']]),
                              status_code=200,
                              headers={'content-type':'application/json'})

            response = s.collections
            assert response['my_collection1']

    def test_collection(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            response = s.catalog

            requests_mock.get(match_url, json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            collection = s.collection('my_collection1')

            assert collection == stac_objects[k]['collection.json']
            assert collection.keywords
            assert collection.version
            assert collection.license
            assert collection.properties
            assert collection.providers[0].name
            assert collection.providers[0].description
            assert collection.providers[0].roles
            assert collection.providers[0].url
            if k == '0.7.0':
                assert collection.extent.spatial
                assert collection.extent.temporal
            else:
                assert collection.extent.spatial.bbox
                assert collection.extent.temporal.interval
                assert collection.summaries
                assert collection.summaries['val'].min == 0
                assert collection.summaries['val'].max == 1

    def test_collection_missing(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            response = s.catalog

            requests_mock.get(match_url,
                              exc=requests.exceptions.HTTPError)

            with pytest.raises(KeyError):
                s.collection('missing_collection')

    def test_collection_without_catalog(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url, json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            response = s.collection('my_collection1')

            assert response == stac_objects[k]['collection.json']

    def test_item(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            if k == '0.9.0':
                requests_mock.get(re.compile(url+'/'), json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            else:
                requests_mock.get(re.compile(url+'/stac'), json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            catalog = s.catalog

            s.catalog #test for non empty catalog

            requests_mock.get(match_url, json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            collection = s.collection('my_collection1')

            requests_mock.get(match_url, json=stac_objects[k]['items.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            itemcollection = s.collection('my_collection1').get_items()
            item = itemcollection.features[0]

            if collection.stac_version == '0.8.0':
                assert itemcollection.links
                assert item.properties.datetime
                assert item.properties.license
                assert item.properties.providers
                assert item.properties.title
                assert item.properties.created
                assert item.properties.updated
            assert itemcollection.type
            assert item.id == 'feature1'
            assert item.stac_version
            assert item.type
            assert item.bbox
            assert item.collection
            assert item.geometry.type
            assert item.geometry.coordinates
            assert item.properties
            assert item.links
            assert item.assets['thumbnail'].href
            assert item.assets['thumbnail'].title
            assert item.assets['thumbnail'].type

    def test_item_id(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            response = s.catalog

            requests_mock.get(match_url, json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            collection = s.collection('my_collection1')

            requests_mock.get(match_url, json=stac_objects[k]['items.json']['features'][0],
                              status_code=200,
                              headers={'content-type':'application/json'})
            response = collection.get_items(item_id='feature1')

            assert response.id == 'feature1'

    def test_item_empty(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)
            requests_mock.get(match_url, json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})
            collection = s.collection('my_collection1')

            requests_mock.get(match_url, json=stac_objects[k]['items.json']['features'][0],
                              status_code=200,
                              headers={'content-type':'application/json'})
            collection['links'].pop(1)
            assert collection.get_items() == stac.ItemCollection({})

    def test_search(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)

            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            response = s.catalog

            requests_mock.get(match_url, json=stac_objects[k]['items.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            response = s.search()

            assert response.features[0].id == 'feature1'

    def test_empty_bbox(self, stac_objects, requests_mock):
        for k in stac_objects:
            s = stac.STAC(url + "/stac" if k != '0.9.0' else url, True)

            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            with pytest.raises(TypeError):
                s.search(filter={"bbox": ""})

            with pytest.raises(TypeError):
                s.search(filter={"bbox": -90})

class TestCli:
    def test_catalog(self, stac_objects, requests_mock, runner):
        for k in stac_objects:
            requests_mock.get(match_url, json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            result = runner.invoke(stac.cli.catalog, ['--url', url + "/stac" if k != '0.9.0' else url])
            assert result.exit_code == 0
            assert 'my_collection1' in result.output

    def test_collection(self, stac_objects, requests_mock, runner):
        for k in stac_objects:
            requests_mock.get(re.compile(url + "/stac" if k != '0.9.0' else url),
                              json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            requests_mock.get(re.compile(url+'/collections'), json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            result = runner.invoke(stac.cli.collection, ['--url', url + "/stac" if k != '0.9.0' else url,
                                                          '--collection-id', 'my_collection1'])
            assert result.exit_code == 0
            assert 'my_collection1' in result.output

    def test_items(self, stac_objects, requests_mock, runner):
        for k in stac_objects:

            requests_mock.get(re.compile(url + "/stac" if k != '0.9.0' else url+'/'), json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            requests_mock.get(re.compile(url+'/collections'), json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            requests_mock.get(re.compile(url+'/collections/my_collection1/items'), json=stac_objects[k]['items.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            result = runner.invoke(stac.cli.items, ['--url', url + "/stac" if k != '0.9.0' else url,
                                                    '--collection-id', 'my_collection1',
                                                    '--limit', 1,
                                                    '--page', 1,
                                                    '--datetime','2016-05-03/2019-01-01',
                                                    '--bbox','-180,-90,180,90'])
            assert result.exit_code == 0
            assert 'feature1' in result.output

    def test_search(self, stac_objects, requests_mock, runner):
        for k in stac_objects:
            requests_mock.get(re.compile(url + "/stac" if k != '0.9.0' else url +'/'),
                              json=stac_objects[k]['catalog.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            requests_mock.get(re.compile(url+'/collections'), json=stac_objects[k]['collection.json'],
                              status_code=200,
                              headers={'content-type':'application/json'})

            requests_mock.post(re.compile(url + "/stac/search" if k != '0.9.0' else url + '/search'),
                               json=stac_objects[k]['items.json'],
                               status_code=200,
                               headers={'content-type':'application/json'})

            intersects = json.dumps(stac_objects[k]['items.json']['features'][0]['geometry'])

            result = runner.invoke(stac.cli.search, ['--url', url + "/stac" if k != '0.9.0' else url,
                                                    '--collections', 'my_collection1',
                                                    '--ids', 'feature1',
                                                    '--intersects', intersects,
                                                    '--limit', 1,
                                                    '--next', 'aaa',
                                                    '--page', 1,
                                                    '--datetime','2016-05-03/2019-01-01',
                                                    '--bbox','-180,-90,180,90'
                                                    ])

            assert result.exit_code == 0
            assert 'feature1' in result.output


if __name__ == '__main__':
    pytest.main(['--color=auto', '--no-cov'])



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Python Client Library for STAC."""

import os

from setuptools import find_packages, setup

readme = open('README.rst').read()

history = open('CHANGES.rst').read()

docs_require = [
    'Sphinx>=2.2',
    'sphinx_rtd_theme',
    'sphinx-copybutton',
]

tests_require = [
    'coverage>=4.5',
    'coveralls>=1.8',
    'pytest>=5.2',
    'pytest-cov>=2.8',
    'pytest-pep8>=1.0',
    'pydocstyle>=4.0',
    'isort>4.3',
    'check-manifest>=0.40',
    'requests-mock>=1.7.0'
]

examples_require = [
    'matplotlib>=3.1',
    'numpy>=1.18'
    'rasterio>=1.1',
]

extras_require = {
    'docs': docs_require,
    'examples': examples_require,
    'oauth': ['requests_oauthlib>=1.3'],
    'tests': tests_require,
    'tqdm': ['tqdm>=4.49.0'],
    'geo': [
        'rasterio>=1.1',
    ]
}

extras_require['all'] = [req for exts, reqs in extras_require.items() for req in reqs]

setup_requires = [
    'pytest-runner>=5.2',
]

install_requires = [
    'Click>=7.0',
    'requests>=2.20',
    'Shapely>=1.7,<2',
    'jsonschema>=3.2',
    'Jinja2>=2.11.1'
]

packages = find_packages()

with open(os.path.join('stac', 'version.py'), 'rt') as fp:
    g = {}
    exec(fp.read(), g)
    version = g['__version__']

setup(
    name='stac.py',
    version=version,
    description=__doc__,
    long_description=readme + '\n\n' + history,
    long_description_content_type = 'text/x-rst',
    keywords=['SpatioTemporal Asset Catalog', 'stac', 'earth-observation', 'geospatial', 'gis'],
    license='MIT',
    author="Brazil Data Cube Team",
    author_email="brazildatacube@inpe.br",
    url='https://github.com/brazil-data-cube/stac.py',
    packages=packages,
    zip_safe=False,
    include_package_data=True,
    platforms='any',
    entry_points={
        'console_scripts': [
            'stac = stac.cli:cli',
        ],
    },
    extras_require=extras_require,
    install_requires=install_requires,
    setup_requires=setup_requires,
    tests_require=tests_require,
    classifiers=[
        'Development Status :: 3 - Alpha',
        'Environment :: Web Environment',
        'Intended Audience :: Education',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python :: 3.7',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Topic :: Scientific/Engineering :: GIS',
    ],
)



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#
"""STAC Catalog module."""

import json

from pkg_resources import resource_string

from .common import Link
from .utils import Utils


class Catalog(dict):
    """The STAC Catalog."""

    def __init__(self, data, validate=False):
        """Initialize instance with dictionary data.

        :param data: Dict with catalog metadata.
        :param validate: true if the Catalog should be validate using its jsonschema. Default is False.
        """
        self._validate = validate
        super(Catalog, self).__init__(data or {})

        self._schema = json.loads(resource_string(__name__, f'jsonschemas/{self.stac_version}/catalog.json'))

        if self._validate:
            Utils.validate(self)

        self._links = [Link(link) for link in self['links']] if 'links' in self else []

    @property
    def stac_version(self):
        """:return: the STAC version."""
        return self['stac_version']

    @property
    def id(self):
        """:return: the catalog identifier."""
        return self['id']

    @property
    def title(self):
        """:return: the catalog title."""
        return self['title'] if 'title' in self else None

    @property
    def description(self):
        """:return: the catalog description."""
        return self['description']

    @property
    def links(self):
        """:return: a list of resources in the catalog."""
        return self._links

    @property
    def schema(self):
        """:return: the Catalog jsonschema."""
        return self._schema



#
# This file is part of Python Client Library for STAC.
# Copyright (C) 2019-2021 INPE.
#
# Python Client Library for STAC is free software; you can redistribute it and/or modify it
# under the terms of the MIT License; see LICENSE file for more details.
#

"""Configuration file for the STAC client library documentation.

The documentation system is based on Sphinx. If you want to know
more about the options to be used for configuration, please, see:

- https://www.sphinx-doc.org/en/master/usage/configuration.html
"""

import stac
import sphinx_rtd_theme


# -- Project information -----------------------------------------------------

project = 'STAC Client'
copyright = '2020, INPE,'
author = 'Brazil Data Cube Team'
release = stac.__version__


# -- General configuration ---------------------------------------------------

# Enabled Sphinx extensions.
extensions = [
    'sphinx.ext.autodoc',
    'sphinx.ext.doctest',
    'sphinx.ext.napoleon',
    'sphinx.ext.todo',
    'sphinx_copybutton',
    'sphinx_rtd_theme',
]

# Paths that contain templates, relative to this directory.
templates_path = ['_templates']

# The language for content autogenerated by Sphinx.
language = 'en_US'

# List of patterns, relative to source directory, that match files and
# directories to ignore when looking for source files.
# This pattern also affects html_static_path and html_extra_path.
exclude_patterns = [
    '_build',
    'Thumbs.db',
    '.DS_Store',
]


# -- Options for HTML output -------------------------------------------------

# The theme to use for HTML and HTML Help pages.
html_theme = 'sphinx_rtd_theme'

html_baseurl = 'https://brazil-data-cube.github.io/'

html_theme_options = {
    'analytics_id': 'XXXXXXXXXX',
    'logo_only': False,
    'display_version': True,
    'prev_next_buttons_location': 'both',
    'style_external_links': True,
    #'vcs_pageview_mode': 'edit',
    #'github_url': 'https://github.com/brazil-data-cube/stac.py',
    'style_nav_header_background': '#2980B9',
    'collapse_navigation': True,
    'sticky_navigation': False,
    'navigation_depth': 3,
    'includehidden': True,
    'titles_only': False
}

html_title = 'STAC Client'

html_context = {
    'display_github': False,
    'github_user': 'brazil-data-cube',
    'github_repo': 'stac.py',
    'last_updated': False,
    #'commit': False,
}

html_show_sourcelink = False

html_logo = './img/logo-bdc.png'

html_favicon = './img/favicon.ico'

html_css_files = [ ]

html_last_updated_fmt = '%b %d, %Y'

html_show_sphinx = False

html_search_language = 'en'

numfig = True

numfig_format = {
    'figure': 'Figure %s -',
    'table': 'Table %s -',
    'code-block': 'Code snippet %s -',
    'section': 'Section %s.'
}

master_doc = 'index'
